/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.charts;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.category.DefaultCategoryDataset;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.StringTokenizer;

public class BarChart extends BaseChart implements Chart {

  public static final String NAME = "bc";

  private BarRenderer renderer;
  private CategoryPlot categoryPlot;
  private DefaultCategoryDataset categoryDataset;
  private CategoryAxis categoryAxis;
  private NumberAxis numberAxis;

  private String rangeMax = null;

  protected BarChart() {
    super();
    initChart();
  }

  protected BarChart(int width, int height) {
    super(width, height);
    initChart();
  }

  protected BarChart(String width, String height, String colors, String rangeMax) {
    this.rangeMax = rangeMax;
    initChart();
    configureDimensions(width, height);
    configureColors(colors);
  }

  private void initChart() {
    configureCategoryDataset();
    configureCategoryAxis();
    configureRenderer();
    configureRangeAxis();
    configureCategoryPlot();
    initDefaultColors();
  }

  protected void configureColors(String colors) {
    try {
      if (colors != null && colors.length() > 0) {
        StringTokenizer stringTokenizer = new StringTokenizer(colors, ",");
        int i = 0;
        while (stringTokenizer.hasMoreTokens()) {
          renderer.setSeriesPaint(i, Color.decode("0x" + stringTokenizer.nextToken()));
          i++;
        }
      }
    }
    catch (Exception e) {
      initDefaultColors();
    }
  }

  private void initDefaultColors() {
    renderer.setSeriesPaint(0, SERIE1_COLOR);
    renderer.setSeriesPaint(1, SERIE2_COLOR);
    renderer.setSeriesPaint(2, SERIE3_COLOR);
    renderer.setSeriesPaint(3, SERIE4_COLOR);
    renderer.setSeriesPaint(4, SERIE5_COLOR);
  }

  private void configureCategoryDataset() {
    categoryDataset = new DefaultCategoryDataset();
  }

  private void configureCategoryAxis() {
    categoryAxis = new CategoryAxis();
    categoryAxis.setVisible(false);
    categoryAxis.setTickLabelsVisible(false);
  }

  private void configureRenderer() {
    renderer = new BarRenderer();
    renderer.setItemMargin(0.05);
    renderer.setDrawBarOutline(false);
  }

  private void configureRangeAxis() {
    numberAxis = new NumberAxis();
    numberAxis.setTickLabelsVisible(false);
    numberAxis.setVisible(false);
    if (this.rangeMax != null && this.rangeMax.length() > 0) {
      double iRangeMax = Double.parseDouble(this.rangeMax);
      numberAxis.setRange(0.0, iRangeMax);
    }
  }

  private void configureCategoryPlot() {
    categoryPlot = new CategoryPlot();
    categoryPlot.setDataset(categoryDataset);
    categoryPlot.setOrientation(PlotOrientation.HORIZONTAL);
    categoryPlot.setDomainAxis(categoryAxis);
    categoryPlot.setRenderer(renderer);
    categoryPlot.setOutlineVisible(false);
    categoryPlot.setRangeGridlinesVisible(false);
    categoryPlot.setRangeAxis(numberAxis);
    categoryPlot.setBackgroundAlpha(0.0f);
  }


  protected BufferedImage getChartImage() throws IOException {
    JFreeChart jfreechart = new JFreeChart(null, TextTitle.DEFAULT_FONT, categoryPlot, false);
    configureChart(jfreechart, false);
    return getBufferedImage(jfreechart);
  }

  public void addMeasure(Double value, String series, String category) {
    categoryDataset.addValue(value, series, category);
  }

  public void addMeasures(String values) {
    if (values != null && values.length() > 0) {
      StringTokenizer st = new StringTokenizer(values, ",");
      int i = 0;
      while (st.hasMoreTokens()) {
        double measure = 0;
        try {
          measure = Double.parseDouble(st.nextToken());
        }
        catch (NumberFormatException e) {
        }
        addMeasure(measure, Integer.toString(i), "1");
        i++;
      }
    }
  }

}
