/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.web.maven;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.io.FileUtils;
import org.sonar.commons.database.DatabaseConnector;
import org.sonar.plugins.api.Plugin;
import org.sonar.plugins.api.PluginsIntrospector;
import org.sonar.web.WebProperties;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

public class MavenRepositoryBuilder {
  public static final String DEPENDENCY_VERSION = "1.0-SNAPSHOT";
  public static final String DEPENDENCY_GROUP_ID = "org.codehaus.sonar.runtime";

  private Configuration configuration;
  private DatabaseConnector databaseConnector;
  private PluginsIntrospector pluginsIntrospector;
  private MavenRepository repository;

  public MavenRepositoryBuilder(Configuration configuration, DatabaseConnector databaseConnector,
                                PluginsIntrospector pluginsIntrospector) {
    this.configuration = configuration;
    this.databaseConnector = databaseConnector;
    this.pluginsIntrospector = pluginsIntrospector;
    this.repository = new MavenRepository(getPublicMavenDirectory());
  }

  protected MavenRepositoryBuilder(DatabaseConnector databaseConnector, MavenRepository repository,
                                   PluginsIntrospector pluginsIntrospector) {
    this.databaseConnector = databaseConnector;
    this.pluginsIntrospector = pluginsIntrospector;
    this.repository = repository;
  }

  protected MavenRepositoryBuilder() {
  }

  public void start() {
    deployJdbcDriver(repository);
    deployPlugins(repository);
    deployRuleExtensions(repository);
  }

  public void stop() {
    if (repository != null) {
      repository.stop();
    }
  }

  private void deployJdbcDriver(MavenRepository repository) {
    deployJdbcDriver(repository, "extensions/jdbc-driver/");
  }

  protected void deployJdbcDriver(MavenRepository repository, String extensionsBasedir) {
    try {
      String dialect = databaseConnector.getDialect().toLowerCase();
      List<File> jars = new ArrayList<File>();
      findJars(extensionsBasedir + dialect + "/", jars);
      repository.deploy(DEPENDENCY_GROUP_ID, "jdbc-driver", DEPENDENCY_VERSION, jars.toArray(new File[]{}));

    } catch (Exception e) {
      throw new RuntimeException("can not deploy the JDBC driver to maven repository", e);
    }
  }

  private void deployPlugins(MavenRepository repository) {
    deployPlugins(repository, "extensions/plugins/");
  }

  protected void deployPlugins(MavenRepository repository, String extensionsBasedir) {
    try {
      List<File> jars = new ArrayList<File>();
      findJars(extensionsBasedir, jars);
      repository.deploy(DEPENDENCY_GROUP_ID, "plugins", DEPENDENCY_VERSION, jars.toArray(new File[]{}));

    } catch (Exception e) {
      throw new RuntimeException("can not deploy plugins to maven repository", e);
    }
  }

  private void deployRuleExtensions(MavenRepository repository) {
    deployRuleExtensions(repository, "extensions/rules/");
  }

  protected void deployRuleExtensions(MavenRepository repository, String extensionsBasedir) {
    try {
      List<File> jars = new ArrayList<File>();
      for (Plugin plugin : pluginsIntrospector.getPlugins()) {
        findJars(extensionsBasedir + plugin.getKey(), jars);
      }
      repository.deploy(DEPENDENCY_GROUP_ID, "rules-extensions", DEPENDENCY_VERSION, jars.toArray(new File[]{}));

    } catch (Exception e) {
      throw new RuntimeException("can not deploy plugins to maven repository", e);
    }
  }

  protected void findJars(String dir, List<File> jars) throws URISyntaxException, IOException {
    Enumeration<URL> dirs = getClass().getClassLoader().getResources(dir);
    if (dirs != null) {
      while (dirs.hasMoreElements()) {
        File dirFile = new File((dirs.nextElement()).toURI());
        for (Object file : FileUtils.listFiles(dirFile, new String[]{"jar"}, true)) {
          jars.add((File) file);
        }
      }

    }
  }

  private File getPublicMavenDirectory() {
    return new File(configuration.getString(WebProperties.ROOT_PUBLIC_DIR), "maven");
  }
}
